package com.me.brickbuster.entity.powerup;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.CircleShape;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.me.brickbuster.entity.Ball;
import com.me.brickbuster.entity.Entity;
import com.me.brickbuster.entity.Paddle;
import com.me.brickbuster.physics.CollisionListener;
import com.me.brickbuster.physics.EntityType;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.PlayState;

public abstract class PowerUp extends Entity implements PhysicsBody, CollisionListener {

	public static final float RADIUS = 1.2f;
	public static final float FALL_SPEED = 15f;

	private Color color;
	private boolean isCaught;

	private Body body;

	public PowerUp(PlayState state, Vector2 pos, Color color) {
		super(state, pos);
		this.color = color;
		createBody();
	}

	@Override
	public void render(SpriteBatch sb, ShapeRenderer sr) {
		sr.begin(ShapeType.Filled);
		sr.setColor(color);
		sr.circle(pos.x, pos.y, RADIUS, 20);
		sr.end();
	}

	@Override
	public void update(float dt) {
		if (isCaught) {
			activate();
			delete();
		}

		if (getY() + RADIUS < 0) {
			delete();
		}
	}

	@Override
	public Body getBody() {
		return body;
	}

	@Override
	public void createBody() {
		BodyDef ballBody = new BodyDef();
		ballBody.type = BodyDef.BodyType.DynamicBody;
		ballBody.position.set(pos);

		CircleShape ballShape = new CircleShape();
		ballShape.setRadius(RADIUS);

		FixtureDef ballFixture = new FixtureDef();
		ballFixture.shape = ballShape;
		ballFixture.isSensor = true;

		ballFixture.filter.categoryBits = EntityType.POWER_UP;
		ballFixture.filter.maskBits = EntityType.PADDLE;

		body = state.world.createBody(ballBody);
		body.createFixture(ballFixture);
		body.setUserData(this);

		body.setLinearVelocity(0f, -FALL_SPEED);

		ballShape.dispose();
	}

	@Override
	public void beginContact(Entity contacted) {
		if (contacted instanceof Paddle || contacted instanceof Ball) {
			isCaught = true;
		}
	}

	@Override
	public void endContact(Entity contacted) {}

	public abstract void activate();

}
