package com.me.brickbuster.utils;

import java.util.Iterator;

public class CompositeIterator<T> implements Iterator<T> {

	private short index = 0;
	private Iterator<? extends T> iterator;
	private Iterable<? extends T>[] iterables;

	private CompositeIterator(Iterable<? extends T>... iterables) {
		this.iterables = iterables;
		this.iterator = iterables[index].iterator();
	}

	@Override
	public boolean hasNext() {
		if (iterator.hasNext()) {
			return true;
		} else {
			if (++index < iterables.length) {
				iterator = iterables[index].iterator();
				return hasNext();
			}
		}
		return false;
	}

	@Override
	public T next() {
		return iterator.next();
	}

	@Override
	public void remove() {
		iterator.remove();
	}

	public static <E> CompositeIterator<E> concat(Iterable<? extends E>... iterables) {
		return new CompositeIterator<E>(iterables);
	}

}
